
/*
  Name: rubrica.java
  Copyright: GPL
  Author: Alessandro Bugatti
  Date: 08/28/07 22:09:10
  Description: Programma che carica un file XML aderente allo schema rubrica.xsd
  e permette di effettuare le seguenti operazioni
  - cercare i dati di un nominativo inserendo il cognome
  - aggiungere un nuovo nominativo
  - eliminare un nominativo
  Le modifiche fatte sui dati vengono riversate nel file XML alla chiusura 
  del programma
 */

import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.FactoryConfigurationError;  
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.Result;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;

import org.xml.sax.SAXException;  
import org.xml.sax.SAXParseException;  

import java.io.File;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;

import org.w3c.dom.Document;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class rubrica {
    
    static Document document; 
    
    public static void menu()
    {
      System.out.println("Rubrica");
      System.out.println("1 - Cerca");
      System.out.println("2 - Aggiungi");
      System.out.println("3 - Elimina");
      System.out.println("0 - Esci");
      System.out.println("Scegli l'opzione: ");
    }
    
    public static String delete(Document doc, String n)
    {
      NodeList personaLst = doc.getElementsByTagName("persona");
      for (int i = 0; i < personaLst.getLength(); i++) 
      {
        Node personaNode = personaLst.item(i);
        Element personaElement = (Element) personaNode;
        NodeList cognomeLst = personaElement.getElementsByTagName("cognome");
        Element cognomeElement = (Element) cognomeLst.item(0);
        if (n.equals(((Node) cognomeElement.getChildNodes().item(0)).getNodeValue())) 
          { 
            personaNode.getParentNode().removeChild(personaNode);
            return n + " eliminato dalla rubrica";
          }
      }
      return n + " non trovato";
    }
    
    public static void insert(Document doc, String n, String c, String i, String t)
    {
      NodeList rootLst = doc.getElementsByTagName("rubrica");
      Node rootNode = rootLst.item(0);
      Element root = (Element) rootNode;
      Element persona = doc.createElement("persona");
      Element nome = doc.createElement("nome");
      nome.insertBefore(doc.createTextNode(n), nome.getFirstChild());
      Element cognome = doc.createElement("cognome");
      cognome.insertBefore(doc.createTextNode(c), cognome.getFirstChild());
      Element indirizzo = doc.createElement("indirizzo");
      indirizzo.insertBefore(doc.createTextNode(i), indirizzo.getFirstChild());
      Element telefono = doc.createElement("telefono");
      telefono.insertBefore(doc.createTextNode(t), telefono.getFirstChild());
      persona.insertBefore(telefono, persona.getFirstChild());
      persona.insertBefore(indirizzo, persona.getFirstChild());
      persona.insertBefore(cognome, persona.getFirstChild());
      persona.insertBefore(nome, persona.getFirstChild());
      root.insertBefore(persona, root.getFirstChild());
    }
    
    public static String find(Document doc, String s)
    {
      NodeList personaLst = doc.getElementsByTagName("persona");
      
      for (int i = 0; i < personaLst.getLength(); i++) 
      {
        Node personaNode = personaLst.item(i);
        Element personaElement = (Element) personaNode;
        NodeList cognomeLst = personaElement.getElementsByTagName("cognome");
        Element cognomeElement = (Element) cognomeLst.item(0);
        if (s.equals(((Node) cognomeElement.getChildNodes().item(0)).getNodeValue())) 
        {
          String information = s + " ";
          NodeList nodeLst = personaElement.getElementsByTagName("nome");
          Element element= (Element) nodeLst.item(0);
          information += ((Node) element.getChildNodes().item(0)).getNodeValue() + " ";
          nodeLst = personaElement.getElementsByTagName("indirizzo");
          element = (Element) nodeLst.item(0);
          information += ((Node) element.getChildNodes().item(0)).getNodeValue() + " ";
          nodeLst = personaElement.getElementsByTagName("telefono");
          element = (Element) nodeLst.item(0);
          information += ((Node) element.getChildNodes().item(0)).getNodeValue();
          return information;
        }
      }  
      return "Non trovato";
    }
    
    public static void writeXmlFile(Document doc, String filename) 
    {
        try {
            // prepara il documento DOM per la scrittura
            Source source = new DOMSource(doc);
    
            // Prepara il file di output
            File file = new File(filename);
            Result result = new StreamResult(file);
    
            // Scrive il documento DOM sul file utilizzando un Transformer
            Transformer xformer = TransformerFactory.newInstance().newTransformer();
            xformer.transform(source, result);
        } catch (TransformerConfigurationException e) {
        } catch (TransformerException e) {
        }
    }
    
    public static void main(String argv[]) throws IOException
    {
        if (argv.length != 1) {
            System.err.println("Usage: java rubrica filename");
            System.exit(1);
        }

        DocumentBuilderFactory factory =
        DocumentBuilderFactory.newInstance();
        try {
           DocumentBuilder builder = factory.newDocumentBuilder();
           document = builder.parse( new File(argv[0]) );
 
        } catch (SAXException sxe) {
           // Errori generati durante la fase di parsing
           Exception  x = sxe;
           if (sxe.getException() != null)
               x = sxe.getException();
           x.printStackTrace();

        } catch (ParserConfigurationException pce) {
            // Errore generato se il parser con alcune opzioni specifiche non 
            // pu essere generato
            pce.printStackTrace();

        } catch (IOException ioe) {
           // Errore di I/O 
           ioe.printStackTrace();
        }
        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
        String s;
        int scelta;
        System.out.println("Root element " + document.getDocumentElement().getNodeName());
        do
        {
          menu();
          s = in.readLine();
          scelta = Integer.valueOf(s).intValue();
          switch(scelta)
          {
            case 1: System.out.println("Inserisci il cognome da cercare");
              s = in.readLine();
              String cercato = find(document,s);
              System.out.println(cercato);
            break;
            case 2: System.out.println("Inserisci il nome");
              String nome = in.readLine();
              System.out.println("Inserisci il cognome");
              String cognome = in.readLine();
              System.out.println("Inserisci l'indirizzo");
              String indirizzo = in.readLine();
              System.out.println("Inserisci il telefono");
              String telefono = in.readLine();
              insert (document, nome, cognome, indirizzo, telefono);
            break;
            case 3:System.out.println("Inserisci il cognome da eliminare");
              s = in.readLine();
              String eliminato = delete(document,s);
              System.out.println(eliminato);
            break;
          }
        }while(scelta != 0);
        writeXmlFile(document,argv[0]);
    } 

}
